from unittest.mock import patch
from django.urls import reverse
from rest_framework import status
from arkindex.dataimport.providers import GitLabProvider
from arkindex.project.tests import FixtureAPITestCase


class TestProviders(FixtureAPITestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.creds = cls.user.credentials.get()
        cls.repo = cls.creds.repos.get()

    def test_init(self):
        glp = GitLabProvider()
        self.assertEqual(glp.url, GitLabProvider.url)

        glp = GitLabProvider(url='something')
        self.assertEqual(glp.url, 'something')

        glp = GitLabProvider(credentials=self.creds)
        self.assertEqual(glp.url, self.creds.provider_url)

        with self.assertRaises(Exception):
            GitLabProvider(credentials='not a OAuthCredentials')

        glp = GitLabProvider(credentials=self.creds, url='something')
        self.assertEqual(glp.url, 'something')

    @patch('arkindex.dataimport.api.Repository.provider_class')
    def test_webhook(self, provider_class):
        self.client.force_login(self.user)
        response = self.client.post(reverse('api:import-hook', kwargs={'pk': self.repo.id}))
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertTrue(provider_class.return_value.handle_webhook.called)
