from pathlib import Path
from django.conf import settings
from ponos.management.base import PonosCommand, FolderArgument
from arkindex.project.argparse import CorpusArgument
from arkindex.documents.acts import ActsImporter
import logging

logging.basicConfig(
    format='%(asctime)s [%(levelname)s] %(message)s',
    level=logging.INFO,
)


class Command(PonosCommand):
    help = "Import surfaces and acts from CSV files"
    docker_image = settings.ARKINDEX_APP_IMAGE
    base_recipe = settings.PONOS_RECIPE

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'csv',
            help='Path to a folder containing CSV files with surface-act associations, named after the volumes.',
            type=FolderArgument(),
        )
        parser.add_argument(
            '--corpus',
            type=CorpusArgument(),
            help='ID or part of the name of a corpus to import acts and surfaces into.',
            required=True,
        )
        parser.add_argument(
            '--folder-type',
            dest='folder_slug',
            help='Slug of an element type for folders in the selected corpus.',
            default='volume',
        )
        parser.add_argument(
            '--act-type',
            dest='act_slug',
            help='Slug of an element type for acts in the selected corpus.',
            default='act',
        )
        parser.add_argument(
            '--surface-type',
            dest='surface_slug',
            help='Slug of an element type for surfaces in the selected corpus.',
            default='surface',
        )

    def run(self, csv, corpus, folder_slug, act_slug, surface_slug, **options):
        csv = Path(csv)
        folder_type = corpus.types.get(slug=folder_slug, folder=True, hidden=False)
        act_type = corpus.types.get(slug=act_slug, folder=False, hidden=False)
        surface_type = corpus.types.get(slug=surface_slug, folder=False, hidden=True)
        for path in csv.rglob('*.csv'):
            try:
                importer = ActsImporter(path, corpus, folder_type, act_type, surface_type)
                importer.run()
            except Exception as e:
                self.stdout.write(self.style.ERROR(str(e)))
