# Generated by Django 2.2 on 2019-09-23 13:03

from django.db import migrations
from enum import Enum


def remove_registries(apps, schema_editor):
    Element = apps.get_model('documents', 'Element')
    ElementPath = apps.get_model('documents', 'ElementPath')

    class OldElementType(Enum):
        Register = 'register'

    # Don't try this at home
    Element._meta.fields[3].enum = OldElementType

    register_ids = list(Element.objects.filter(type='register').values_list('id', flat=True))
    if not register_ids:
        return

    print('Removing {} registries'.format(len(register_ids)))
    # Remove registry from child paths without removing the parent volume
    for register_id in register_ids:
        schema_editor.execute(
            'UPDATE documents_elementpath SET path = array_remove(path, %s) WHERE path @> ARRAY[%s]::uuid[]',
            params=[register_id, register_id],
        )
    # Remove registry parents
    ElementPath.objects.filter(id__in=register_ids).delete()
    # Remove registries
    Element.objects.filter(id__in=register_ids).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0018_page_act_proxies'),
    ]

    operations = [
        migrations.RunPython(
            remove_registries,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
    ]
