# Generated by Django 2.2 on 2019-10-22 10:44

from django.db import migrations
from django.db.models import Prefetch
from arkindex_common.enums import MetaType


def rename_elements(apps, schema_editor):
    """
    Update elements with folio metadatas to use their folios as names
    and save the previous element name as a metadata
    """
    Element = apps.get_model('documents', 'Element')
    MetaData = apps.get_model('documents', 'MetaData')

    qs = Element.objects \
        .filter(metadatas__name='folio') \
        .only('id', 'name') \
        .prefetch_related(
            Prefetch(
                'metadatas',
                queryset=MetaData.objects.filter(name='folio'),
                to_attr='folio_metadatas',
            ),
            Prefetch(
                'metadatas',
                queryset=MetaData.objects.filter(name='name'),
                to_attr='name_metadatas',
            ),
        ) \
        .distinct()

    if not qs.exists():
        return

    new_metadatas = []
    updated_elements = []
    for element in qs:
        if len(element.folio_metadatas) > 1:
            print('Element {!s} has multiple folios; skipping'.format(element.id))

        if element.name_metadatas:
            index = max(md.index for md in element.name_metadatas) + 1
        else:
            index = 0

        new_metadatas.append(
            MetaData(
                element_id=element.id,
                name='name',
                value=element.name,
                type=MetaType.Text,
                index=index,
            )
        )
        element.name = element.folio_metadatas[0].value
        updated_elements.append(element)

    print('Creating {} metadatas…'.format(len(new_metadatas)))
    MetaData.objects.bulk_create(new_metadatas, batch_size=1000)
    print('Renaming {} elements…'.format(len(updated_elements)))
    Element.objects.bulk_update(updated_elements, ['name'], batch_size=1000)


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0024_rename_new_type'),
    ]

    operations = [
        migrations.RunPython(
            code=rename_elements,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        )
    ]
