from django.urls import reverse
from rest_framework import status
from arkindex.project.tests import FixtureAPITestCase


class TestElementAnnotationListSerializer(FixtureAPITestCase):

    def test_normal_list(self):
        page = self.corpus.elements.get(name='Volume 1, page 1r')
        response = self.client.get(reverse('api:element-annotation-list', kwargs={'pk': page.id}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        annotation_list = response.json()

        self.assertIn('@context', annotation_list)
        self.assertIn('@id', annotation_list)
        self.assertIn('@type', annotation_list)
        self.assertIn('resources', annotation_list)

        self.assertEqual(annotation_list['@type'], 'sc:AnnotationList')
        self.assertEqual(annotation_list['@context'], 'http://iiif.io/api/presentation/2/context.json')
        self.assertEqual(len(annotation_list['resources']), 3)

        for annotation in annotation_list['resources']:
            self.assertIn('@type', annotation)
            self.assertIn('@id', annotation)
            self.assertIn('on', annotation)
            self.assertIn('motivation', annotation)
            self.assertIn('resource', annotation)

            self.assertEqual(annotation['@type'], 'oa:Annotation')
            self.assertEqual(annotation['motivation'], 'sc:painting')

            resource = annotation['resource']
            self.assertIn('@id', resource)
            self.assertIn('@type', resource)
            self.assertIn('format', resource)
            self.assertIn('chars', resource)

            self.assertEqual(resource['format'], 'text/plain')
            self.assertEqual(resource['@type'], 'cnt:ContentAsText')
            self.assertIn(resource['chars'], ['PARIS', 'ROY', 'DATUM'])

    def test_empty_list(self):
        # An annotation list with nothing in it
        response = self.client.get(reverse('api:element-annotation-list', kwargs={
            'pk': self.corpus.elements.get(name='Volume 2, page 2r').id
        }))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        annotation_list = response.json()

        self.assertEqual(len(annotation_list['resources']), 0)
