from unittest.mock import patch
from arkindex_common.enums import MetaType
from arkindex.project.tests import FixtureAPITestCase
from arkindex.project.elastic import ESElement
from arkindex.documents.dates import DateType, InterpretedDate


class TestESDocuments(FixtureAPITestCase):

    @patch('arkindex.documents.models.MetaData.get_dates')
    def test_build_search_index_dates(self, get_dates_mock):
        element = self.corpus.elements.create(
            type=self.corpus.types.get(slug='act'),
            name='sister act',
        )
        element.metadatas.create(type=MetaType.Date, name='date', value='something')

        # Test Upper bound
        get_dates_mock.return_value = [InterpretedDate(1420, 5, type=DateType.Upper), ]
        date_range = ESElement.from_model(element).to_dict().get('date_range')
        self.assertDictEqual(date_range, {'lt': '1420-05||+1M'})

        # Test Lower bound
        get_dates_mock.return_value = [InterpretedDate(1418, type=DateType.Lower), ]
        date_range = ESElement.from_model(element).to_dict().get('date_range')
        self.assertDictEqual(date_range, {'gte': '1418'})

        # Test with both Lower and Upper bound
        get_dates_mock.return_value = [
            InterpretedDate(1418, type=DateType.Lower),
            InterpretedDate(1428, type=DateType.Upper),
        ]
        date_range = ESElement.from_model(element).to_dict().get('date_range')
        self.assertDictEqual(date_range, {'gte': '1418', 'lt': '1428||+1y'})

        # Test an exact date
        get_dates_mock.return_value = [InterpretedDate(1666, 2, 3), ]
        date_range = ESElement.from_model(element).to_dict().get('date_range')
        self.assertDictEqual(date_range, {'gte': '1666-02-03', 'lt': '1666-02-03||+1d'})
